
package jworms.game;

import java.awt.geom.Point2D;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInputStream;

public abstract class GameMovingObject extends GameObject{
  /** Short average motion. */
  private Point2D.Float shortAverageMotion = new Point2D.Float();
  /** Long average motion. */
  private Point2D.Float longAverageMotion = new Point2D.Float();
  /** Absolute long average motion. */
  private Point2D.Float absoluteLongAverageMotion = new Point2D.Float();

  /**
   * Constructor for MovingObject.
   */
  public GameMovingObject(boolean isHologram){
    super(isHologram);
  }

  /**
   * Starts the move of the object.
   */
  public final void doMove(){
    // remember last position
    Point2D.Float lastPosition = new Point2D.Float(getPosition().x, getPosition().y);

    move();

    // movement vector
    Point2D.Float delta = new Point2D.Float(getPosition().x - lastPosition.x, getPosition().y - lastPosition.y);

    shortAverageMotion.x = shortAverageMotion.x * 0.9f + delta.x * 0.1f;
    shortAverageMotion.y = shortAverageMotion.y * 0.9f + delta.y * 0.1f;

    longAverageMotion.x = longAverageMotion.x * 0.98f + delta.x * 0.02f;
    longAverageMotion.y = longAverageMotion.y * 0.98f + delta.y * 0.02f;

    absoluteLongAverageMotion.x = absoluteLongAverageMotion.x * 0.98f + Math.abs(delta.x) * 0.02f;
    absoluteLongAverageMotion.y = absoluteLongAverageMotion.y * 0.98f + Math.abs(delta.y) * 0.02f;
  }

  /**
   * Does the actual move.
   */
  protected abstract void move();

  /**
   * Returns the short average motion of the object.
   *
   * @return short average motion
   */
  public Point2D.Float getShortAverageMotion(){
    return shortAverageMotion;
  }

  /**
   * Returns the long average motion of the object.
   *
   * @return long average motion
   */
  public Point2D.Float getLongAverageMotion(){
    return longAverageMotion;
  }

  /**
   * Returns the absolute long average motion of the object.
   *
   * @return absolute long average motion
   */
  public Point2D.Float getAbsoluteLongAverageMotion(){
    return absoluteLongAverageMotion;
  }

// network
  public void sendUpdate(DataOutputStream out) throws IOException{
    out.writeFloat(shortAverageMotion.x);
    out.writeFloat(shortAverageMotion.y);
    out.writeFloat(longAverageMotion.x);
    out.writeFloat(longAverageMotion.y);
    out.writeFloat(absoluteLongAverageMotion.x);
    out.writeFloat(absoluteLongAverageMotion.y);
  }

  public void recieveUpdate(DataInputStream in) throws IOException{
    shortAverageMotion.x = in.readFloat();
    shortAverageMotion.y = in.readFloat();
    longAverageMotion.x = in.readFloat();
    longAverageMotion.y = in.readFloat();
    absoluteLongAverageMotion.x = in.readFloat();
    absoluteLongAverageMotion.y = in.readFloat();
  }
}
