
package jworms.game;

import jworms.game.weapons.Weapon;
import jworms.openGL.SmallTexture;
import org.lwjgl.Display;
import org.lwjgl.opengl.GL;

import java.awt.*;

public class PlayerUI{
// static
  /** Texture for health bar. */
  private static SmallTexture textureHealthBar = null;
  /** Texture for aim cross. */
  private static SmallTexture textureAimCross = null;

  /**
   * Loads textures and sound.
   */
  public static void init(){
    // load images
    try{
      textureHealthBar = new SmallTexture("images/player/health_bar.png", GL.GL_LINEAR);
      textureAimCross = new SmallTexture("images/player/aim.png", GL.GL_LINEAR);
    } catch(Exception e){
      System.out.println("Error in BallUI during loading images.");
      e.printStackTrace();
    }
  }

// non static
  /** The associated player. */
  private Player player = null;
  /** GameField */
  private GameField gameField = null;

  /**
   * Creates a PlayerUI for the given player.
   *
   * @param player player
   */
  public PlayerUI(Player player, GameField gameField){
    this.player = player;
    this.gameField = gameField;
  }

  /**
   * Draws the health bar, aim cross and player name.
   */
  public void draw(){
    Point position = gameField.fieldToScreen((int) player.getInterpolatedPosition().x, (int) player.getInterpolatedPosition().y);
    Weapon weapon = player.getWeapon();

    // draw health bar
    Point positionHealthBar = new Point(position.x - textureHealthBar.getImage().getWidth() / 2 + 2, position.y + 20);
    textureHealthBar.draw(positionHealthBar.x, positionHealthBar.y);

    int healthWidth = Math.max(0, Math.min((int) (25 * player.getHealth()), 25));
    GL.glDisable(GL.GL_TEXTURE_2D);
    GL.glBegin(GL.GL_QUADS);
    {
      GL.glColor3ub((byte) 200, (byte) 255, (byte) 82);
      GL.glVertex2i(positionHealthBar.x + 2, positionHealthBar.y + 2);
      GL.glVertex2i(positionHealthBar.x + 2 + healthWidth, positionHealthBar.y + 2);
      GL.glColor3ub((byte) 85, (byte) 145, (byte) 47);
      GL.glVertex2i(positionHealthBar.x + 2 + healthWidth, positionHealthBar.y + 2 + 3);
      GL.glVertex2i(positionHealthBar.x + 2, positionHealthBar.y + 2 + 3);
      GL.glColor3f(1.0f, 1.0f, 1.0f);
    }
    GL.glEnd();
    GL.glEnable(GL.GL_TEXTURE_2D);

    // draw aim cross
    if(weapon != null && player.isAiming()){
      Point center = weapon.getCenter(player);
      Point positionAimCross = gameField.fieldToScreen(center.x, center.y);

      int radius = 40;
      int dx = (int) (Math.cos(player.getAimingAngle()) * radius);
      int dy = -(int) (Math.sin(player.getAimingAngle()) * radius);
      positionAimCross.x += dx * (player.getLookDirection() == Player.DIRECTION_RIGHT ? 1 : -1);
      positionAimCross.y += dy;

      positionAimCross.x -= textureAimCross.getImage().getWidth() / 2;
      positionAimCross.y -= textureAimCross.getImage().getHeight() / 2;

      float angle = (float) (player.getAimingAngle() * 180 / Math.PI) * (player.getLookDirection() == Player.DIRECTION_RIGHT ? 1 : -1);
      textureAimCross.draw(positionAimCross.x, positionAimCross.y, 1, 1, angle);
    }

    // draw charging bar
    if(player.isCharging()){
      int width = 200;
      int height = 25;
      int x = (Display.getWidth() - width) / 2;
      int y = Display.getHeight() - height - 50;

      int chargingWidth = (int) (width * player.getChargingValue());

      GL.glDisable(GL.GL_TEXTURE_2D);
      GL.glLineWidth(1);
      GL.glBegin(GL.GL_LINE_STRIP);
      {
        GL.glColor3f(0.0f, 0.0f, 0.0f);
        GL.glVertex2i(x - 1, y);
        GL.glVertex2i(x + width, y);
        GL.glVertex2i(x + width, y + height + 1);
        GL.glVertex2i(x - 1, y + height + 1);
        GL.glVertex2i(x - 1, y);
      }
      GL.glEnd();
      GL.glBegin(GL.GL_QUADS);
      {
        GL.glColor4f(0.0f, 0.0f, 0.0f, 0.5f);
        GL.glVertex2i(x, y);
        GL.glVertex2i(x + width, y);
        GL.glVertex2i(x + width, y + height);
        GL.glVertex2i(x, y + height);
      }
      GL.glEnd();
      GL.glBegin(GL.GL_QUADS);
      {
        GL.glColor3ub((byte) 200, (byte) 255, (byte) 82);
        GL.glVertex2i(x, y);
        GL.glVertex2i(x + chargingWidth, y);
        GL.glColor3ub((byte) 85, (byte) 145, (byte) 47);
        GL.glVertex2i(x + chargingWidth, y + height);
        GL.glVertex2i(x, y + height);
        GL.glColor3f(1.0f, 1.0f, 1.0f);
      }
      GL.glEnd();
      GL.glEnable(GL.GL_TEXTURE_2D);
    }

    // draw weapon
    if(weapon != null) weapon.drawSymbol(player);
  }

  /**
   * Free all resources associated with this object.
   */
  public void dispose(){
  }
}
