
package jworms.game.explosion;

import jworms.game.GameField;

import java.awt.geom.Point2D;

/**
 * Part of an explosion, for example a cloud or fire ring.
 */
public abstract class ExplosionObject{
  /** Game field. */
  protected GameField gameField = null;
  /** Position. */
  protected Point2D.Float position = null;
  /** Motion. */
  protected Point2D.Float motion = null;
  /** Heat of the explosion object. */
  protected float heat = 1;

  /** Start time. */
  protected long startTime = 0;
  /** Maximum life time */
  protected float maxLife = 1;
  /** Seconds of livetime. */
  protected float lifeSeconds = 0;

  /**
   * Creates an explosion object.
   *
   * @param position position
   * @param motion motion
   */
  public ExplosionObject(GameField gameField, Point2D.Float position, Point2D.Float motion){
    this.gameField = gameField;
    this.position = position;
    this.motion = motion;

    startTime = System.currentTimeMillis();
  }

  /**
   * Returns true if the live time of the object is finished.
   *
   * @return true if finished
   */
  public abstract boolean isFinished();

  public Point2D.Float getPosition(){
    return position;
  }

  public void doMove(){
    lifeSeconds = (System.currentTimeMillis() - startTime) / 1000.f;

    move();
  }

  /**
   * Does the actual move.
   */
  protected abstract void move();

  /**
   * Draws the object.
   */
  public abstract void draw();

  /**
   * Returns the heat of the object.
   *
   * @return heat heat 0..1
   */
  public float getHeat(){
    return heat;
  }

  /**
   * Sets the heat of the object.
   *
   * @param heat heat 0..1
   */
  public void setHeat(float heat){
    this.heat = heat;
  }
}
