
package jworms.game.weapons;

import jworms.game.GameConfiguration;
import jworms.game.Player;
import jworms.openGL.SmallTexture;
import org.lwjgl.Display;
import org.lwjgl.input.Keyboard;
import org.lwjgl.opengl.GL;

/**
 * Shows available weapons and lets the user choose one.
 */
public class WeaponViewer{
// static
  /** Texture for the weapons panel. */
  protected static SmallTexture weaponsPanel = null;

  /**
   * Loads textures and sound.
   */
  public static void init(){
    // load images
    try{
      weaponsPanel = new SmallTexture("images/weapons/weapon_viewer.png", GL.GL_NEAREST);
    } catch(Exception e){
      System.out.println("Error in WeaponViewer during loading images.");
      e.printStackTrace();
    }
  }

// non static
  /** The associated player. */
  private Player player = null;

  /** Weapons */
  private Weapon weapons[] = null;

  /**
   * Creates a WeaponViewer for the given player.
   *
   * @param player player
   */
  public WeaponViewer(Player player){
    this.player = player;

    // create 9 fields for weapons
    weapons = new Weapon[9];
  }

  /**
   * Returns the weapon at the given index.
   *
   * @param index index 0..8
   * @return weapon
   */
  public Weapon getWeapon(int index){
    return weapons[index];
  }

  /**
   * Sets the weapon at the given index.
   *
   * @param index index
   * @param weapon weapon
   */
  public void setWeapon(int index, Weapon weapon){
    weapons[index] = weapon;

    if(index == 0) player.setWeapon(weapon);
  }

  public void move(){
    if(!player.hasKeyFocus()) return;

    // did the user choose a weapon
    int weaponIndex = -1;

    if(Keyboard.isKeyDown(GameConfiguration.getKey(GameConfiguration.ACTION_WEAPON_1))) weaponIndex = 0;
    if(Keyboard.isKeyDown(GameConfiguration.getKey(GameConfiguration.ACTION_WEAPON_2))) weaponIndex = 1;
    if(Keyboard.isKeyDown(GameConfiguration.getKey(GameConfiguration.ACTION_WEAPON_3))) weaponIndex = 2;
    if(Keyboard.isKeyDown(GameConfiguration.getKey(GameConfiguration.ACTION_WEAPON_4))) weaponIndex = 3;
    if(Keyboard.isKeyDown(GameConfiguration.getKey(GameConfiguration.ACTION_WEAPON_5))) weaponIndex = 4;
    if(Keyboard.isKeyDown(GameConfiguration.getKey(GameConfiguration.ACTION_WEAPON_6))) weaponIndex = 5;
    if(Keyboard.isKeyDown(GameConfiguration.getKey(GameConfiguration.ACTION_WEAPON_7))) weaponIndex = 6;
    if(Keyboard.isKeyDown(GameConfiguration.getKey(GameConfiguration.ACTION_WEAPON_8))) weaponIndex = 7;
    if(Keyboard.isKeyDown(GameConfiguration.getKey(GameConfiguration.ACTION_WEAPON_9))) weaponIndex = 8;

    if(weaponIndex == -1) return;

    Weapon weapon = getWeapon(weaponIndex);
    if(weapon == null) return;

    // give the player another weapon
    if(player.getWeapon() != weapon) player.setWeapon(weapon);
  }

  /**
   * Draws the weapons panel in the lower righ corner of the display.
   */
  public void draw(){
    int x = Display.getWidth() - weaponsPanel.getImage().getWidth() - 10;
    int y = Display.getHeight() - weaponsPanel.getImage().getHeight() - 10;

    GL.glDisable(GL.GL_BLEND);
    weaponsPanel.draw(x, y);
    GL.glEnable(GL.GL_BLEND);
  }
}
