
package jworms.network;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

public class GameInfo {
// static
  /**
   * Reads all game info properties from stream and returns a game info object with this properties.
   *
   * @param in input stream
   * @return game info
   * @throws IOException thrown if reading fails
   */
  public static GameInfo createFromStream(DataInputStream in) throws IOException {
    GameInfo gameInfo = new GameInfo();
    gameInfo.fromStream(in);
    return gameInfo;
  }

// non static
  /** Name of the game. */
  private String name;
  /** Current count of players in this game. */
  private int playerCount;
  /** Maximal count of players in this game. */
  private int maxPlayerCount;
  /** Port the game is running on. */
  private int port;

  /**
   * Create an empty game info to read all properties from stream.
   */
  private GameInfo(){
  }

  /**
   * Creates a game info.
   *
   * @param name name of the game
   * @param playerCount current count of players
   * @param maxPlayerCount maximal count of players
   * @param port port the game is running on
   */
  public GameInfo(String name, int playerCount, int maxPlayerCount, int port){
    this.name = name;
    this.playerCount = playerCount;
    this.maxPlayerCount = maxPlayerCount;
    this.port = port;
  }

  /**
   * Returns the name of the game.
   *
   * @return name of the game
   */
  public String getName() {
    return name;
  }

  /**
   * Returns the current count of players.
   *
   * @return current count of players
   */
  public int getPlayerCount() {
    return playerCount;
  }

  /**
   * Returns the maximal count of players.
   *
   * @return maximal count of players
   */
  public int getMaxPlayerCount() {
    return maxPlayerCount;
  }

  /**
   * Returns the port the game is running on.
   *
   * @return port the game is running on
   */
  public int getPort() {
    return port;
  }

  /**
   * Writes all game info properties to stream.
   *
   * @param out output stream
   * @throws IOException thrown if writing fails
   */
  public void toStream(DataOutputStream out) throws IOException {
    out.writeUTF(name);
    out.writeInt(playerCount);
    out.writeInt(maxPlayerCount);
    out.writeInt(port);
  }

  /**
   * Reads all game info properties from stream.
   *
   * @param in input stream
   * @throws IOException thrown if reading fails
   */
  public void fromStream(DataInputStream in) throws IOException {
    name = in.readUTF();
    playerCount = in.readInt();
    maxPlayerCount = in.readInt();
    port = in.readInt();
  }

  public String toString(){
    return name + " on port " + port + ", players: " + playerCount + " / " + maxPlayerCount;
  }
}
