
package jworms.network;

import java.io.*;
import java.net.*;

public class UDPGetGames extends Thread {
  /** Parent. */
  private UDPNetwork udpNetwork;
  /** Port listening on. */
  private int port;
  /** Socket to receive and send packets. */
  DatagramSocket socket;

  /**
   * Creates a thread listening to the hosts to get informed about all running game.
   *
   * @param udpNetwork the UDPNetwork
   * @param port the port listening on
   */
  public UDPGetGames(UDPNetwork udpNetwork, int port){
    this.udpNetwork = udpNetwork;
    this.port = port;

    setDaemon(true);

    // run thread
    start();
  }

  public void run(){
    try{
      // listening
      System.out.println("GetGames is listening on port " + port + "...");
      socket = new DatagramSocket(port-2);

      byte[] buf = new byte[256];

      DatagramPacket p = new DatagramPacket(buf, buf.length);

      while(!isInterrupted()){
        socket.receive(p);

//        System.out.println("GetGames: " + new String(p.getData()));
        ByteArrayInputStream byteIn = new ByteArrayInputStream(p.getData());
        DataInputStream in = new DataInputStream(byteIn);
        if(UDPNetwork.readSignature(in)){
          if(in.readByte() == UDPNetwork.TYPE_GAME_INFO){
            GameInfo gameInfo = GameInfo.createFromStream(in);
            udpNetwork.foundGame(gameInfo);
          }
        }
      }

      socket.close();
    } catch(SocketException e){
      System.out.println("Error listening on UDP port " + port + ": " + e.getMessage());
    } catch(IOException e){
      System.out.println("Error receiving message: " + e.getMessage());
    }
  }

  /**
   * Sends the "get games"-request to all hosts.
   *
   * @throws IOException thrown if sending request failed
   * @throws UnknownHostException thrown if ... no that's impossible
   */
  public void sendRequest() throws IOException, UnknownHostException {
    ByteArrayOutputStream out = new ByteArrayOutputStream(256);
    DataOutputStream dataOut = new DataOutputStream(out);

    // write header
    UDPNetwork.writeSignature(dataOut);
    dataOut.writeByte(UDPNetwork.TYPE_GET_GAMES);

    System.out.println("Sending \"looking for games\" request...");
    DatagramSocket socket = new DatagramSocket();

    InetAddress iaddr = InetAddress.getByName("255.255.255.255");
    byte[] bytes = out.toByteArray();
    DatagramPacket p = new DatagramPacket(bytes, bytes.length, iaddr, port);

    socket.send(p);
  }

  public void interrupt() {
    super.interrupt();
    socket.close();
  }
}
