
package jworms.network;

import java.io.*;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketException;

public class UDPHost extends Thread {
  /** Game info of the running game. */
  private GameInfo gameInfo;
  /** Port listening on. */
  private int port;
  /** Socket to receive and send packets. */
  DatagramSocket socket;

  /**
   * Creates an UDPHost to tell other that here runs a game.
   *
   * @param gameInfo the game info of the current running game
   * @param port the port listening on
   */
  public UDPHost(GameInfo gameInfo, int port){
    this.gameInfo = gameInfo;
    this.port = port;

    setDaemon(true);

    // run thread
    start();
  }

  public void run(){
    try{
      System.out.println("Host is listening on port " + port + "...");
      socket = new DatagramSocket(port);

      byte[] buf = new byte[256];

      DatagramPacket p = new DatagramPacket(buf, buf.length);

      while(!isInterrupted()){
        socket.receive(p);

//        System.out.println("Host: " + new String(p.getData()));

        ByteArrayInputStream byteIn = new ByteArrayInputStream(p.getData());
        DataInputStream in = new DataInputStream(byteIn);
        if(UDPNetwork.readSignature(in)){
          if(in.readByte() == UDPNetwork.TYPE_GET_GAMES){
            sendReply(socket, p);
          }
        }
      }

      socket.close();
    } catch(SocketException e){
      System.out.println("Error listening on UDP port " + port + ": " + e.getMessage());
    } catch(IOException e){
      System.out.println("Error receiving message: " + e.getMessage());
    }
  }

  /**
   * Sends the game info to the inquirer.
   *
   * @param s socket
   * @param p received packet
   */
  private void sendReply(DatagramSocket s, DatagramPacket p){
    try{
      ByteArrayOutputStream out = new ByteArrayOutputStream(256);
      DataOutputStream dataOut = new DataOutputStream(out);

      // write header
      UDPNetwork.writeSignature(dataOut);
      dataOut.writeByte(UDPNetwork.TYPE_GAME_INFO);
      gameInfo.toStream(dataOut);

      // send the packet
      System.out.println("Sending game info");
      p.setData(out.toByteArray());
      p.setPort(port-2);
      s.send(p);
      p.setPort(port);
    } catch(IOException e){
      System.out.println("Error sending reply: " + e.getMessage());
    }
  }

  public void interrupt() {
    super.interrupt();
    socket.close();
  }
}
