
package jworms.openAL;

import org.lwjgl.openal.AL;
import org.lwjgl.test.openal.WaveData;

import java.nio.IntBuffer;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public class OpenALWave{
// static properties
  /** ArrayList of waves to be created. */
  private static HashMap genWaves = new HashMap();
  private static IntBuffer buffers = null;

// wave properties
  /** The path of the wave file. */
  private String path;
  /** The id or name of the wave file. */
  int id;

  /**
   * Load a wave file. Don't use this methode directly. Use OpenALWave.create(String path) instead
   * to avoid multiple loading of one of the same wave file.
   *
   * @param path the file name of the wave file
   */
  private OpenALWave(String path){
    this.path = path;

    // add to list to load wave later
    genWaves.put(path, this);
  }

  /**
   * Use this methode to load wave files.
   *
   * @param path path the file name of the wave file
   * @return the wave object
   */
  public static OpenALWave create(String path){
    Object obj = genWaves.get(path);
    if(obj != null)
      return (OpenALWave) obj;
    else
      return new OpenALWave(path);
  }

  /**
   * Generates all loaded waves.
   * Must be called before waves can be used with OpenAL.
   */
  public static void alGenWaves(){
    // create buffer
    buffers = OpenALUtils.createIntBuffer(genWaves.size());
    AL.alGenBuffers(buffers);
    OpenALUtils.checkError();

    Iterator it = genWaves.entrySet().iterator();

    int i = 0;
    while(it.hasNext()){
      OpenALWave openALWave = (OpenALWave) (((Map.Entry) it.next()).getValue());

      //load wave data
      System.out.println("Loading wave file: " + openALWave.path);
      WaveData wavefile = WaveData.create(openALWave.path);

      openALWave.id = buffers.get(i);

      //copy to buffers
      AL.alBufferData(
              openALWave.id,
              wavefile.format,
              wavefile.data,
              wavefile.data.capacity(),
              wavefile.samplerate);
      OpenALUtils.checkError();

      //unload file again
      wavefile.dispose();

      i++;
    }
  }

  /**
   * Deletes all generatet buffers.
   */
  public static void alDeleteWaves(){
    if(buffers != null){
      AL.alDeleteBuffers(buffers);
      buffers = null;
    }
  }

  public String toString(){
    return "File name: \"" + path + "\", ID: " + id;
  }
}
