
package jworms.openGL;

import java.awt.*;

class BigTextureTile implements Comparable{
  /** Horizontal index of this texture within a bigger one. */
  public int tileX;
  /** Vertical index of this texture within a bigger one. */
  public int tileY;
  /** Position of this texture within the bigger image.. */
  public Rectangle location = null;

  /** OpenGL texture. */
  public OpenGLTextureInternal textureInternal = null;

  /** Time/ Frame of last use. */
  public long lastUse = 0;
  /** True if the texture is valid (up to date). */
  public boolean isValid = false;

  /**
   * Creates a texture tile with the given position, width and height.
   *
   * @param x x position within the bigger texture
   * @param x y position within the bigger texture
   * @param width width
   * @param height height
   */
  public BigTextureTile(int tileX, int tileY, int x, int y, int width, int height){
    this.tileX = tileX;
    this.tileY = tileY;
    location = new Rectangle(x, y, width, height);
  }

  /**
   * Compares two textures for last use time.
   *
   * @param o BigTextureTile to compare with
   * @return -1 of o is newer, 0 if equal, 1 if o is older
   */
  public int compareTo(Object o){
    if(!(o instanceof BigTextureTile)) return 0;

    BigTextureTile texture = ((BigTextureTile) o);

    if(!texture.isValid){
      if(isValid) return 1; else return 0;
    }

    if(lastUse < texture.lastUse) return -1;
    if(lastUse > texture.lastUse) return 1;
    return 0;
  }
}
