
package tutorial;

import jworms.openAL.OpenALListener;
import jworms.openAL.OpenALSource;
import jworms.openAL.OpenALUtils;
import jworms.openAL.OpenALWave;
import jworms.openGL.BigTexture;
import jworms.openGL.BitmapFont;
import jworms.openGL.OpenGLTextureInternal;
import jworms.openGL.OpenGLUtils;
import org.lwjgl.Display;
import org.lwjgl.Sys;
import org.lwjgl.input.Keyboard;
import org.lwjgl.input.Mouse;
import org.lwjgl.openal.AL;
import org.lwjgl.opengl.GL;
import org.lwjgl.opengl.Window;

import java.awt.*;

/**
 * Demonstrates the usage of jworms.openGL.BigTexture and jworms.openAL.*
 *
 * usage: java tutorial.BigTextureTutorial
 *        (close with Escape or Alt+F4)
 */
public class BigTextureTutorial{
  // bitmap font to display the frame rate
  private BitmapFont font = null;

  // landscape texture (big texture)
  BigTexture textureLandscape = null;

  // Sound source.
  private OpenALSource soundSource = new OpenALSource();
  // Hit sound
  private OpenALWave soundHit = null;

  // upper left corner of the texture on the screen
  private Point position = new Point();
  // motion of the texture
  private Point motion = new Point(-7, -5);

  // frames per second
  private long framesPerSecond = 0;

  public static void main(String[] arguments){
    new BigTextureTutorial();
  }

  /**
   * Initialized OpenGL fullscreen and renders a big landscape texture.
   */
  public BigTextureTutorial(){
    // initialize OpenGL
    OpenGLUtils.initFullscreen("JWorms");
    // initialize OpenAL
    OpenALUtils.alInit();

    // load the landscape textur, font and sound
    try{
      font = new BitmapFont("tutorial/font.dat");
      textureLandscape = new BigTexture("tutorial/landscape.png",
                                        BigTexture.FORMAT_RGB, false);

      soundHit = OpenALWave.create("tutorial/hit.wav");
    } catch(Exception e){
      System.out.println("Failed starting game due to ");
      e.printStackTrace();
      System.exit(-1);
    }

    // generate all loaded textures and sounds
    OpenGLTextureInternal.glGenTextures();
    OpenALWave.alGenWaves();

    // update the frames counter every second
    long framesCounter = 0;
    double lastTime = getTime();

    // close with Alt+F4 or Escape
    while(!OpenGLUtils.isCloseRequested() && !Keyboard.isKeyDown(Keyboard.KEY_ESCAPE)){
      Mouse.poll();
      Keyboard.poll();

      motion();
      // render the landscape
      render();

      // update frames counter every second (uses high resolution timer of LWJGL)
      if(getTime() - lastTime > 1){
        framesPerSecond = Math.round(framesCounter / (getTime() - lastTime));

        framesCounter = 0;
        lastTime = getTime();
      }

      // update internal window state
      Window.update();
      // swap buffers
      Window.paint();

      framesCounter++;
    }

    // dispose OpenGL
    OpenGLUtils.destroy();
    // dispose OpenAL
    OpenALUtils.alExit();
  }

  /**
   * Returns the time of the hres timer (high resolution time) in seconds.
   *
   * @return time in seconds
   */
  private double getTime(){
    return ((double) Sys.getTime()) / Sys.getTimerResolution();
  }

  /**
   * Lets the texture scroll. (just for fun)
   */
  private void motion(){
    position.x += motion.x;
    position.y += motion.y;

    // mirroring motion
    if(position.x < Display.getWidth() - textureLandscape.getImage().getWidth()){
      position.x = Display.getWidth() - textureLandscape.getImage().getWidth();
      motion.x = -motion.x;
      playSound();
    }
    if(position.x > 0){
      position.x = 0;
      motion.x = -motion.x;
      playSound();
    }
    if(position.y < Display.getHeight() - textureLandscape.getImage().getHeight()){
      position.y = Display.getHeight() - textureLandscape.getImage().getHeight();
      motion.y = -motion.y;
      playSound();
    }
    if(position.y > 0){
      position.y = 0;
      motion.y = -motion.y;
      playSound();
    }
  }

  /**
   * Renders the big texture with jworms.OpenGL and LWJGL OpenGL.
   */
  private void render(){
    // to draw the map we don't need blend or alpha test
    GL.glDisable(GL.GL_BLEND);
    GL.glDisable(GL.GL_ALPHA_TEST);

    textureLandscape.draw(position.x, position.y);

    GL.glEnable(GL.GL_BLEND);

    // show frames per second
    font.drawString("frames per second: " + framesPerSecond, 20, 20);

    // update openAL listener position
    OpenALListener.setPos(position.x + Display.getWidth() / 2,
                          position.y + Display.getHeight() / 2, 100);
  }

  /**
   * Plays a sound with jworms.OpenAL and LWJGL OpenAL.
   */
  private void playSound(){
    if(soundSource.bind()){
      soundSource.stop();
      soundSource.setPos(position.x, position.y, 0);
      soundSource.setPropf(AL.AL_GAIN, 1);
      soundSource.play(soundHit);
    }
  }
}
